#!/usr/bin/python

import flup.server.fcgi, urlparse, requests, json, os, time, errno, hashlib

# Config file must contain, eg.:
# { "path-vhost": "/var/www/m/blog.zorinaq.com",
#   "recaptcha-secret": "<Google_reCAPTCHA_secret_key>",
#   "enforce-recaptcha": true,
#   "secret-passwd": "<...>",
#   "secret-user": "<...>" }
cfg_file = '/etc/hablog.config'
cfg_mtime = None
cfg = {}

class UserException(Exception):
    pass

def reload_config():
    global cfg_mtime, cfg
    x = os.path.getmtime(cfg_file)
    if x != cfg_mtime:
        cfg_mtime = x
        cfg = json.loads(file(cfg_file).read())

def record_comment(user, comment, post_id, silently_ignore, seed, ip, ua):
    trusted = 0
    if 'secret-passwd' in cfg and 'secret-user' in cfg:
        if user == cfg['secret-passwd']:
            user = cfg['secret-user']
            trusted = 1
    if '\r' in user or '\n' in user:
        raise UserException('Malformed username')
    # All post IDs must contain a dash.
    if '-' not in post_id or '/' in post_id:
        raise UserException('Malformed post-id')
    comment = comment.replace('\r', '')
    p_commdir = cfg['path-vhost'] + '/db/' + post_id
    if not os.path.isdir(p_commdir):
        raise UserException('Invalid post-id')
    if not silently_ignore:
        # deterministically generate the comment ID
        cmt_id = hashlib.\
            sha1('%s\0%s\0%s\0%s' % (post_id, seed, user, comment)).\
            hexdigest()[:32]
        p_comm = '%s/%d.%s' % (p_commdir, int(time.time()), cmt_id)
        dat = json.dumps({ \
            "ip": ip,
            "user-agent": ua,
            "user": user,
            "comment": comment,
            "removed": 0,
            "trusted": trusted,
            })
        f = file(p_comm, 'w')
        f.write(dat)
        f.close()

def hablog_app(environ, start_response):
    log = environ['wsgi.errors']
    silently_ignore = False
    try:
        reload_config()
        resp = ""
        #for var in environ.keys():
        #    resp += '%s: %s\n' % (var, environ[var])
        obj = urlparse.parse_qs(environ['wsgi.input'].read())
        # obj contains:
        # { 'user': ['...'],
        #   'comment': ['...'],
        #   'url': ['...'],
        #   'post-id': ['...'],
        #   'seed': ['...'],
        #   'g-recaptcha-response': ['...'] }
        if 'user' not in obj:
            raise UserException('username was empty')
        if 'comment' not in obj:
            raise UserException('comment was empty')
        if 'url' in obj:
            silently_ignore = True
        if 'post-id' not in obj:
            raise Exception('post-id was not specified')
        if 'seed' not in obj:
            raise Exception('seed was not specified')
        if cfg['enforce-recaptcha']:
            if 'g-recaptcha-response' not in obj:
                raise UserException('reCAPTCHA box was not checked')
            payload = { 'secret': cfg['recaptcha-secret'],
                    'response': obj['g-recaptcha-response'][0],
                    'remoteip': environ['REMOTE_ADDR'] }
            #log.write('reCAPTCHA payload: %s\n' % (repr(payload)))
            r = requests.post('https://www.google.com/recaptcha/api/siteverify',
                    data=payload)
            if r.status_code != 200:
                raise Exception('reCAPTCHA response status %d' % r.status_code)
            re_resp = r.json() # will raise an exception on error
            #log.write('reCAPTCHA response: %s\n' % (repr(re_resp)))
            if 'success' not in re_resp:
                raise Exception('reCAPTCHA response weird: %s' % repr(re_resp))
            if re_resp['success'] != True:
                raise UserException('Comment rejected. You seem to be a ' +
                    'robot, sorry!')
        record_comment(obj['user'][0], obj['comment'][0], obj['post-id'][0],
                silently_ignore, obj['seed'][0], environ['REMOTE_ADDR'],
                environ['HTTP_USER_AGENT'])
        start_response('303 See Other', [('Location', environ['HTTP_REFERER']),
            ('Content-Type', 'text/plain')])
        return [resp]
    except UserException as e:
        start_response('500 Internal Error', [('Content-Type', 'text/plain')])
        return ['%s' % e] # user-facing error
    except Exception as e:
        start_response('500 Internal Error', [('Content-Type', 'text/plain')])
        log.write('hablog.fcgi error: %s\n' % e)
        return ['hablog.fcgi error\n']

if __name__ == '__main__':
    os.umask(022) # it is convenient to make comment files world-readable
    flup.server.fcgi.WSGIServer(hablog_app).run()
